@@ -1,5 +1,5 @@
 from types import MethodType
-from typing import Union, Any, List, Callable, Dict, Optional
+from typing import Tuple, Union, Any, List, Callable, Dict, Optional
 from functools import partial, wraps
 from easydict import EasyDict
 import copy
@@ -9,7 +9,7 @@ import numbers
 import logging
 import enum
 import time
-import traceback
+
 from ding.utils import ENV_MANAGER_REGISTRY, import_module, one_time_warning
 from ding.envs.env.base_env import BaseEnvTimestep
 from ding.utils.time_helper import WatchDog
@@ -24,47 +24,15 @@ class EnvState(enum.IntEnum):
     ERROR = 5
 
 
-def retry_wrapper(func: Callable = None, max_retry: int = 10, waiting_time: float = 0.1) -> Callable:
-    """
-    Overview:
-        Retry the function until exceeding the maximum retry times.
-    """
-
-    if func is None:
-        return partial(retry_wrapper, max_retry=max_retry)
-
-    if max_retry == 1:
-        return func
-
-    @wraps(func)
-    def wrapper(*args, **kwargs):
-        exceptions = []
-        for _ in range(max_retry):
-            try:
-                ret = func(*args, **kwargs)
-                return ret
-            except BaseException as e:
-                exceptions.append(e)
-                time.sleep(waiting_time)
-        logging.error("Function {} has exceeded max retries({})".format(func, max_retry))
-        runtime_error = RuntimeError(
-            "Function {} has exceeded max retries({}), and the latest exception is: {}".format(
-                func, max_retry, repr(exceptions[-1])
-            )
-        )
-        runtime_error.__traceback__ = exceptions[-1].__traceback__
-        raise runtime_error
-
-    return wrapper
-
-
-def timeout_wrapper(func: Callable = None, timeout: int = 10) -> Callable:
+def timeout_wrapper(func: Callable = None, timeout: Optional[int] = None) -> Callable:
     """
     Overview:
         Watch the function that must be finihsed within a period of time. If timeout, raise the captured error.
     """
     if func is None:
         return partial(timeout_wrapper, timeout=timeout)
+    if timeout is None:
+        return func
 
     windows_flag = platform.system().lower() == 'windows'
     if windows_flag:
@@ -95,9 +63,10 @@ class BaseEnvManager(object):
     Overview:
         Create a BaseEnvManager to manage multiple environments.
     Interfaces:
-        reset, step, seed, close, enable_save_replay, launch, env_info, default_config
+        reset, step, seed, close, enable_save_replay, launch, default_config
     Properties:
-        env_num, ready_obs, done, method_name_list，active_env
+        env_num, ready_obs, done, method_name_list, active_env,
+        observation_space, action_space, reward_space
     """
 
     @classmethod
@@ -109,9 +78,10 @@ class BaseEnvManager(object):
     config = dict(
         episode_num=float("inf"),
         max_retry=1,
-        step_timeout=60,
+        retry_type='reset',
         auto_reset=True,
-        reset_timeout=60,
+        step_timeout=None,
+        reset_timeout=None,
         retry_waiting_time=0.1,
     )
 
@@ -134,13 +104,20 @@ class BaseEnvManager(object):
         self._env_replay_path = None
         # env_ref is used to acquire some common attributes of env, like obs_shape and act_shape
         self._env_ref = self._env_fn[0]()
+        self._env_ref.reset()
+        self._observation_space = self._env_ref.observation_space
+        self._action_space = self._env_ref.action_space
+        self._reward_space = self._env_ref.reward_space
+        self._env_ref.close()
         self._env_states = {i: EnvState.VOID for i in range(self._env_num)}
         self._env_seed = {i: None for i in range(self._env_num)}
 
         self._episode_num = self._cfg.episode_num
-        self._max_retry = self._cfg.max_retry
-        self._step_timeout = self._cfg.step_timeout
+        self._max_retry = max(self._cfg.max_retry, 1)
         self._auto_reset = self._cfg.auto_reset
+        self._retry_type = self._cfg.retry_type
+        assert self._retry_type in ['reset', 'renew'], self._retry_type
+        self._step_timeout = self._cfg.step_timeout
         self._reset_timeout = self._cfg.reset_timeout
         self._retry_waiting_time = self._cfg.retry_waiting_time
 
@@ -148,6 +125,18 @@ class BaseEnvManager(object):
     def env_num(self) -> int:
         return self._env_num
 
+    @property
+    def observation_space(self) -> 'gym.spaces.Space':  # noqa
+        return self._observation_space
+
+    @property
+    def action_space(self) -> 'gym.spaces.Space':  # noqa
+        return self._action_space
+
+    @property
+    def reward_space(self) -> 'gym.spaces.Space':  # noqa
+        return self._reward_space
+
     @property
     def ready_obs(self) -> Dict[int, Any]:
         """
@@ -214,6 +203,7 @@ class BaseEnvManager(object):
         self._envs = [e() for e in self._env_fn]
         # env_ref is used to acquire some common attributes of env, like obs_shape and act_shape
         self._env_ref = self._envs[0]
+        self._env_ref.reset()
         assert len(self._envs) == self._env_num
         self._reset_param = {i: {} for i in range(self.env_num)}
         self._env_states = {i: EnvState.INIT for i in range(self.env_num)}
@@ -256,7 +246,6 @@ class BaseEnvManager(object):
 
     def _reset(self, env_id: int) -> None:
 
-        @retry_wrapper(max_retry=self._max_retry, waiting_time=self._retry_waiting_time)
         @timeout_wrapper(timeout=self._reset_timeout)
         def reset_fn():
             # if self._reset_param[env_id] is None, just reset specific env, not pass reset param
@@ -266,14 +255,32 @@ class BaseEnvManager(object):
             else:
                 return self._envs[env_id].reset()
 
-        try:
-            obs = reset_fn()
-        except Exception as e:
-            self._env_states[env_id] = EnvState.ERROR
-            self.close()
-            raise e
-        self._ready_obs[env_id] = obs
-        self._env_states[env_id] = EnvState.RUN
+        exceptions = []
+        for _ in range(self._max_retry):
+            try:
+                obs = reset_fn()
+                self._ready_obs[env_id] = obs
+                self._env_states[env_id] = EnvState.RUN
+                return
+            except BaseException as e:
+                if self._retry_type == 'renew':
+                    err_env = self._envs[env_id]
+                    err_env.close()
+                    self._envs[env_id] = self._env_fn[env_id]()
+                exceptions.append(e)
+                time.sleep(self._retry_waiting_time)
+                continue
+
+        self._env_states[env_id] = EnvState.ERROR
+        self.close()
+        logging.error("Env {} reset has exceeded max retries({})".format(env_id, self._max_retry))
+        runtime_error = RuntimeError(
+            "Env {} reset has exceeded max retries({}), and the latest exception is: {}".format(
+                env_id, self._max_retry, repr(exceptions[-1])
+            )
+        )
+        runtime_error.__traceback__ = exceptions[-1].__traceback__
+        raise runtime_error
 
     def step(self, actions: Dict[int, Any]) -> Dict[int, namedtuple]:
         """
@@ -312,16 +319,25 @@ class BaseEnvManager(object):
 
     def _step(self, env_id: int, act: Any) -> namedtuple:
 
-        @retry_wrapper(max_retry=self._max_retry, waiting_time=self._retry_waiting_time)
         @timeout_wrapper(timeout=self._step_timeout)
         def step_fn():
             return self._envs[env_id].step(act)
 
-        try:
-            return step_fn()
-        except Exception as e:
-            self._env_states[env_id] = EnvState.ERROR
-            raise e
+        exceptions = []
+        for _ in range(self._max_retry):
+            try:
+                return step_fn()
+            except BaseException as e:
+                exceptions.append(e)
+        self._env_states[env_id] = EnvState.ERROR
+        logging.error("Env {} step has exceeded max retries({})".format(env_id, self._max_retry))
+        runtime_error = RuntimeError(
+            "Env {} step has exceeded max retries({}), and the latest exception is: {}".format(
+                env_id, self._max_retry, repr(exceptions[-1])
+            )
+        )
+        runtime_error.__traceback__ = exceptions[-1].__traceback__
+        raise runtime_error
 
     def seed(self, seed: Union[Dict[int, int], List[int], int], dynamic_seed: bool = None) -> None:
         """
@@ -372,15 +388,6 @@ class BaseEnvManager(object):
             self._env_states[i] = EnvState.VOID
         self._closed = True
 
-    def env_info(self) -> namedtuple:
-        """
-        Overview:
-            Get one env's info, for example, action space, observation space, reward space, etc.
-        Returnns:
-            - info (:obj:`namedtuple`): Usually a namedtuple ``BaseEnvInfo``, each element is ``EnvElementInfo``.
-        """
-        return self._env_ref.info()
-
 
 def create_env_manager(manager_cfg: dict, env_fn: List[Callable]) -> BaseEnvManager:
     r"""
