@@ -15,7 +15,9 @@ from easydict import EasyDict
 from types import MethodType
 
 from ding.utils import PropagatingThread, LockContextType, LockContext, ENV_MANAGER_REGISTRY
-from .base_env_manager import BaseEnvManager, EnvState, retry_wrapper, timeout_wrapper
+from .base_env_manager import BaseEnvManager, EnvState, timeout_wrapper
+from ding.envs.env.base_env import BaseEnvTimestep
+import gym
 
 _NTYPE_TO_CTYPE = {
     np.bool_: ctypes.c_bool,
@@ -67,9 +69,9 @@ class ShmBuffer():
             - src_arr (:obj:`np.ndarray`): array to fill the buffer.
         """
         assert isinstance(src_arr, np.ndarray), type(src_arr)
-        dst_arr = np.frombuffer(self.buffer.get_obj(), dtype=self.dtype).reshape(self.shape)
+        dst_arr = np.frombuffer(self.buffer.get_obj(), dtype=self.dtype)
         with self.buffer.get_lock():
-            np.copyto(dst_arr, src_arr)
+            np.copyto(dst_arr, src_arr.flatten())
 
     def get(self) -> np.ndarray:
         """
@@ -78,8 +80,7 @@ class ShmBuffer():
         Return:
             - copy_data (:obj:`np.ndarray`): A copy of the data stored in the buffer.
         """
-        arr = np.frombuffer(self.buffer.get_obj(), dtype=self.dtype).reshape(self.shape)
-        return arr.copy()
+        return np.frombuffer(self.buffer.get_obj(), dtype=self.dtype).reshape(self.shape)
 
 
 class ShmBufferContainer(object):
@@ -157,15 +158,16 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
         Create an AsyncSubprocessEnvManager to manage multiple environments.
         Each Environment is run by a respective subprocess.
     Interfaces:
-        seed, launch, ready_obs, step, reset, env_info，active_env
+        seed, launch, ready_obs, step, reset, active_env
     """
 
     config = dict(
         episode_num=float("inf"),
         max_retry=5,
-        step_timeout=60,
+        step_timeout=None,
         auto_reset=True,
-        reset_timeout=60,
+        retry_type='reset',
+        reset_timeout=None,
         retry_waiting_time=0.1,
         # subprocess specified args
         shared_memory=True,
@@ -173,6 +175,7 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
         wait_num=2,
         step_wait_timeout=0.01,
         connect_timeout=60,
+        reset_inplace=False,
     )
 
     def __init__(
@@ -200,13 +203,13 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
 
         self._lock = LockContext(LockContextType.THREAD_LOCK)
         self._connect_timeout = self._cfg.connect_timeout
-        self._connect_timeout = np.max([self._connect_timeout, self._step_timeout + 0.5, self._reset_timeout + 0.5])
         self._async_args = {
             'step': {
                 'wait_num': min(self._wait_num, self._env_num),
                 'timeout': self._step_wait_timeout
             }
         }
+        self._reset_inplace = self._cfg.reset_inplace
 
     def _create_state(self) -> None:
         r"""
@@ -216,11 +219,22 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
         self._env_episode_count = {env_id: 0 for env_id in range(self.env_num)}
         self._ready_obs = {env_id: None for env_id in range(self.env_num)}
         self._env_ref = self._env_fn[0]()
+        self._env_ref.reset()
         self._reset_param = {i: {} for i in range(self.env_num)}
         if self._shared_memory:
-            obs_space = self._env_ref.info().obs_space
-            shape = obs_space.shape
-            dtype = np.dtype(obs_space.value['dtype']) if obs_space.value is not None else np.dtype(np.float32)
+            obs_space = self._env_ref.observation_space
+            self._env_ref.close()
+            if isinstance(obs_space, list):
+                # for multi_agent case
+                if isinstance(obs_space[0], gym.spaces.Dict):
+                    shape = {k: (len(obs_space), ) + v.shape for k, v in obs_space[0].items()}
+                    dtype = list(obs_space[0].values())[0].dtype
+                else:
+                    shape = (len(obs_space), ) + obs_space[0].shape
+                    dtype = obs_space[0].dtype
+            else:
+                shape = obs_space.shape
+                dtype = obs_space.dtype
             self._obs_buffers = {env_id: ShmBufferContainer(dtype, shape) for env_id in range(self.env_num)}
         else:
             self._obs_buffers = {env_id: None for env_id in range(self.env_num)}
@@ -233,8 +247,8 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
 
     def _create_env_subprocess(self, env_id):
         # start a new one
-        self._pipe_parents[env_id], self._pipe_children[env_id] = Pipe()
         ctx = get_context(self._context)
+        self._pipe_parents[env_id], self._pipe_children[env_id] = ctx.Pipe()
         self._subprocesses[env_id] = ctx.Process(
             # target=self.worker_fn,
             target=self.worker_fn_robust,
@@ -246,7 +260,7 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
                 self.method_name_list,
                 self._reset_timeout,
                 self._step_timeout,
-                self._max_retry,
+                self._reset_inplace,
             ),
             daemon=True,
             name='subprocess_env_manager{}_{}'.format(env_id, time.time())
@@ -310,10 +324,6 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
                 value is the cooresponding reset parameters.
         """
         self._check_closed()
-        # clear previous info
-        for env_id in self._waiting_env['step']:
-            self._pipe_parents[env_id].recv()
-        self._waiting_env['step'].clear()
 
         if reset_param is None:
             reset_env_list = [env_id for env_id in range(self._env_num)]
@@ -322,6 +332,12 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
             for env_id in reset_param:
                 self._reset_param[env_id] = reset_param[env_id]
 
+        # clear previous info
+        for env_id in reset_env_list:
+            if env_id in self._waiting_env['step']:
+                self._pipe_parents[env_id].recv()
+                self._waiting_env['step'].remove(env_id)
+
         sleep_count = 0
         while any([self._env_states[i] == EnvState.RESET for i in reset_env_list]):
             if sleep_count % 1000 == 0:
@@ -345,7 +361,7 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
                     ret = self._pipe_parents[env_id].recv()
                     self._check_data({env_id: ret})
                     self._env_seed[env_id] = None  # seed only use once
-                except Exception as e:
+                except BaseException as e:
                     logging.warning("subprocess reset set seed failed, ignore and continue...")
             reset_thread = PropagatingThread(target=self._reset, args=(env_id, ))
             reset_thread.daemon = True
@@ -358,26 +374,19 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
 
     def _reset(self, env_id: int) -> None:
 
-        @retry_wrapper(max_retry=self._max_retry, waiting_time=self._retry_waiting_time)
         def reset_fn():
             if self._pipe_parents[env_id].poll():
                 recv_data = self._pipe_parents[env_id].recv()
-                raise Exception("unread data left before sending to the pipe: {}".format(repr(recv_data)))
+                raise RuntimeError("unread data left before sending to the pipe: {}".format(repr(recv_data)))
             # if self._reset_param[env_id] is None, just reset specific env, not pass reset param
             if self._reset_param[env_id] is not None:
                 assert isinstance(self._reset_param[env_id], dict), type(self._reset_param[env_id])
                 self._pipe_parents[env_id].send(['reset', [], self._reset_param[env_id]])
             else:
-                self._pipe_parents[env_id].send(['reset', [], {}])
+                self._pipe_parents[env_id].send(['reset', [], None])
 
             if not self._pipe_parents[env_id].poll(self._connect_timeout):
-                # terminate the old subprocess
-                self._pipe_parents[env_id].close()
-                if self._subprocesses[env_id].is_alive():
-                    self._subprocesses[env_id].terminate()
-                # reset the subprocess
-                self._create_env_subprocess(env_id)
-                raise Exception("env reset timeout")  # Leave it to retry_wrapper to try again
+                raise ConnectionError("env reset connection timeout")  # Leave it to try again
 
             obs = self._pipe_parents[env_id].recv()
             self._check_data({env_id: obs}, close=False)
@@ -387,16 +396,32 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
             self._env_states[env_id] = EnvState.RUN
             self._ready_obs[env_id] = obs
 
-        try:
-            reset_fn()
-        except Exception as e:
-            logging.error('VEC_ENV_MANAGER: env {} reset error'.format(env_id))
-            logging.error('\nEnv Process Reset Exception:\n' + ''.join(traceback.format_tb(e.__traceback__)) + repr(e))
-            if self._closed:  # exception cased by main thread closing parent_remote
+        exceptions = []
+        for _ in range(self._max_retry):
+            try:
+                reset_fn()
                 return
-            else:
-                self.close()
-                raise e
+            except BaseException as e:
+                if self._retry_type == 'renew' or isinstance(e, pickle.UnpicklingError):
+                    self._pipe_parents[env_id].close()
+                    if self._subprocesses[env_id].is_alive():
+                        self._subprocesses[env_id].terminate()
+                    self._create_env_subprocess(env_id)
+                exceptions.append(e)
+                time.sleep(self._retry_waiting_time)
+
+        logging.error("Env {} reset has exceeded max retries({})".format(env_id, self._max_retry))
+        runtime_error = RuntimeError(
+            "Env {} reset has exceeded max retries({}), and the latest exception is: {}".format(
+                env_id, self._max_retry, repr(exceptions[-1])
+            )
+        )
+        runtime_error.__traceback__ = exceptions[-1].__traceback__
+        if self._closed:  # exception cased by main thread closing parent_remote
+            return
+        else:
+            self.close()
+            raise runtime_error
 
     def step(self, actions: Dict[int, Any]) -> Dict[int, namedtuple]:
         """
@@ -428,7 +453,7 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
                    )
 
         for env_id, act in actions.items():
-            self._pipe_parents[env_id].send(['step', [act], {}])
+            self._pipe_parents[env_id].send(['step', [act], None])
 
         timesteps = {}
         step_args = self._async_args['step']
@@ -441,7 +466,17 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
             ready_conn, ready_ids = AsyncSubprocessEnvManager.wait(rest_conn, min(wait_num, len(rest_conn)), timeout)
             cur_ready_env_ids = [cur_rest_env_ids[env_id] for env_id in ready_ids]
             assert len(cur_ready_env_ids) == len(ready_conn)
-            timesteps.update({env_id: p.recv() for env_id, p in zip(cur_ready_env_ids, ready_conn)})
+            # timesteps.update({env_id: p.recv() for env_id, p in zip(cur_ready_env_ids, ready_conn)})
+            for env_id, p in zip(cur_ready_env_ids, ready_conn):
+                try:
+                    timesteps.update({env_id: p.recv()})
+                except pickle.UnpicklingError as e:
+                    timestep = BaseEnvTimestep(None, None, None, {'abnormal': True})
+                    timesteps.update({env_id: timestep})
+                    self._pipe_parents[env_id].close()
+                    if self._subprocesses[env_id].is_alive():
+                        self._subprocesses[env_id].terminate()
+                    self._create_env_subprocess(env_id)
             self._check_data(timesteps)
             ready_env_ids += cur_ready_env_ids
             cur_rest_env_ids = list(set(cur_rest_env_ids).difference(set(cur_ready_env_ids)))
@@ -467,10 +502,14 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
             if timestep.done:
                 self._env_episode_count[env_id] += 1
                 if self._env_episode_count[env_id] < self._episode_num and self._auto_reset:
-                    self._env_states[env_id] = EnvState.RESET
-                    reset_thread = PropagatingThread(target=self._reset, args=(env_id, ), name='regular_reset')
-                    reset_thread.daemon = True
-                    reset_thread.start()
+                    if self._reset_inplace:  # reset in subprocess at once
+                        self._env_states[env_id] = EnvState.RUN
+                        self._ready_obs[env_id] = timestep.obs
+                    else:
+                        self._env_states[env_id] = EnvState.RESET
+                        reset_thread = PropagatingThread(target=self._reset, args=(env_id, ), name='regular_reset')
+                        reset_thread.daemon = True
+                        reset_thread.start()
                 else:
                     self._env_states[env_id] = EnvState.DONE
             else:
@@ -482,8 +521,12 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
     # A more robust version is used by default. But this one is also preserved.
     @staticmethod
     def worker_fn(
-            p: connection.Connection, c: connection.Connection, env_fn_wrapper: 'CloudPickleWrapper',
-            obs_buffer: ShmBuffer, method_name_list: list
+            p: connection.Connection,
+            c: connection.Connection,
+            env_fn_wrapper: 'CloudPickleWrapper',
+            obs_buffer: ShmBuffer,
+            method_name_list: list,
+            reset_inplace: bool = False,
     ) -> None:  # noqa
         """
         Overview:
@@ -509,6 +552,9 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
                             if is_abnormal_timestep(timestep):
                                 ret = timestep
                             else:
+                                if reset_inplace and timestep.done:
+                                    obs = env.reset()
+                                    timestep = timestep._replace(obs=obs)
                                 if obs_buffer is not None:
                                     obs_buffer.fill(timestep.obs)
                                     timestep = timestep._replace(obs=None)
@@ -546,9 +592,9 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
             env_fn_wrapper,
             obs_buffer,
             method_name_list,
-            reset_timeout=60,
-            step_timeout=60,
-            max_retry=1
+            reset_timeout=None,
+            step_timeout=None,
+            reset_inplace=False,
     ) -> None:
         """
         Overview:
@@ -559,13 +605,15 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
         env = env_fn()
         parent.close()
 
-        @retry_wrapper(max_retry=max_retry)
         @timeout_wrapper(timeout=step_timeout)
         def step_fn(*args, **kwargs):
             timestep = env.step(*args, **kwargs)
             if is_abnormal_timestep(timestep):
                 ret = timestep
             else:
+                if reset_inplace and timestep.done:
+                    obs = env.reset()
+                    timestep = timestep._replace(obs=obs)
                 if obs_buffer is not None:
                     obs_buffer.fill(timestep.obs)
                     timestep = timestep._replace(obs=None)
@@ -581,7 +629,7 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
                     obs_buffer.fill(ret)
                     ret = None
                 return ret
-            except Exception as e:
+            except BaseException as e:
                 env.close()
                 raise e
 
@@ -596,8 +644,10 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
                     ret = getattr(env, args[0])
                 elif cmd in method_name_list:
                     if cmd == 'step':
-                        ret = step_fn(*args, **kwargs)
+                        ret = step_fn(*args)
                     elif cmd == 'reset':
+                        if kwargs is None:
+                            kwargs = {}
                         ret = reset_fn(*args, **kwargs)
                     elif args is None and kwargs is None:
                         ret = getattr(env, cmd)()
@@ -606,8 +656,8 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
                 else:
                     raise KeyError("not support env cmd: {}".format(cmd))
                 child.send(ret)
-            except Exception as e:
-                # print("Sub env '{}' error when executing {}".format(str(env), cmd))
+            except BaseException as e:
+                logging.debug("Sub env '{}' error when executing {}".format(str(env), cmd))
                 # when there are some errors in env, worker_fn will send the errors to env manager
                 # directly send error to another process will lose the stack trace, so we create a new Exception
                 child.send(
@@ -620,7 +670,7 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
     def _check_data(self, data: Dict, close: bool = True) -> None:
         exceptions = []
         for i, d in data.items():
-            if isinstance(d, Exception):
+            if isinstance(d, BaseException):
                 self._env_states[i] = EnvState.ERROR
                 exceptions.append(d)
         # when receiving env Exception, env manager will safely close and raise this Exception to caller
@@ -670,7 +720,9 @@ class AsyncSubprocessEnvManager(BaseEnvManager):
         self._env_ref.close()
         for _, p in self._pipe_parents.items():
             p.send(['close', None, None])
-        for _, p in self._pipe_parents.items():
+        for env_id, p in self._pipe_parents.items():
+            if not p.poll(5):
+                continue
             p.recv()
         for i in range(self._env_num):
             self._env_states[i] = EnvState.VOID
@@ -714,9 +766,10 @@ class SyncSubprocessEnvManager(AsyncSubprocessEnvManager):
     config = dict(
         episode_num=float("inf"),
         max_retry=5,
-        step_timeout=60,
+        step_timeout=None,
         auto_reset=True,
-        reset_timeout=60,
+        reset_timeout=None,
+        retry_type='reset',
         retry_waiting_time=0.1,
         # subprocess specified args
         shared_memory=True,
@@ -724,17 +777,9 @@ class SyncSubprocessEnvManager(AsyncSubprocessEnvManager):
         wait_num=float("inf"),  # inf mean all the environments
         step_wait_timeout=None,
         connect_timeout=60,
-        force_reproducibility=False,
+        reset_inplace=False,  # if reset_inplace=True in SyncSubprocessEnvManager, the interaction can be reproducible.
     )
 
-    def __init__(
-            self,
-            env_fn: List[Callable],
-            cfg: EasyDict = EasyDict({}),
-    ) -> None:
-        super(SyncSubprocessEnvManager, self).__init__(env_fn, cfg)
-        self._force_reproducibility = self._cfg.force_reproducibility
-
     def step(self, actions: Dict[int, Any]) -> Dict[int, namedtuple]:
         """
         Overview:
@@ -763,17 +808,30 @@ class SyncSubprocessEnvManager(AsyncSubprocessEnvManager):
                         for env_id in env_ids}
                    )
         for env_id, act in actions.items():
-            self._pipe_parents[env_id].send(['step', [act], {}])
+            # it is necessary to set kwargs as None for saving cost of serialization in some env like cartpole,
+            # and step method never uses kwargs in known envs.
+            self._pipe_parents[env_id].send(['step', [act], None])
 
         # ===     This part is different from async one.     ===
         # === Because operate in this way is more efficient. ===
         timesteps = {}
         ready_conn = [self._pipe_parents[env_id] for env_id in env_ids]
-        timesteps.update({env_id: p.recv() for env_id, p in zip(env_ids, ready_conn)})
+        # timesteps.update({env_id: p.recv() for env_id, p in zip(env_ids, ready_conn)})
+        for env_id, p in zip(env_ids, ready_conn):
+            try:
+                timesteps.update({env_id: p.recv()})
+            except pickle.UnpicklingError as e:
+                timestep = BaseEnvTimestep(None, None, None, {'abnormal': True})
+                timesteps.update({env_id: timestep})
+                self._pipe_parents[env_id].close()
+                if self._subprocesses[env_id].is_alive():
+                    self._subprocesses[env_id].terminate()
+                self._create_env_subprocess(env_id)
         self._check_data(timesteps)
         # ======================================================
 
         if self._shared_memory:
+            # TODO(nyz) optimize sync shm
             for i, (env_id, timestep) in enumerate(timesteps.items()):
                 timesteps[env_id] = timestep._replace(obs=self._obs_buffers[env_id].get())
         for env_id, timestep in timesteps.items():
@@ -783,12 +841,14 @@ class SyncSubprocessEnvManager(AsyncSubprocessEnvManager):
             if timestep.done:
                 self._env_episode_count[env_id] += 1
                 if self._env_episode_count[env_id] < self._episode_num and self._auto_reset:
-                    self._env_states[env_id] = EnvState.RESET
-                    reset_thread = PropagatingThread(target=self._reset, args=(env_id, ), name='regular_reset')
-                    reset_thread.daemon = True
-                    reset_thread.start()
-                    if self._force_reproducibility:
-                        reset_thread.join()
+                    if self._reset_inplace:  # reset in subprocess at once
+                        self._env_states[env_id] = EnvState.RUN
+                        self._ready_obs[env_id] = timestep.obs
+                    else:
+                        self._env_states[env_id] = EnvState.RESET
+                        reset_thread = PropagatingThread(target=self._reset, args=(env_id, ), name='regular_reset')
+                        reset_thread.daemon = True
+                        reset_thread.start()
                 else:
                     self._env_states[env_id] = EnvState.DONE
             else:
