@@ -1,7 +1,7 @@
 # general
 from .q_learning import DQN, RainbowDQN, QRDQN, IQN, DRQN, C51DQN
+from .qac import QAC, DiscreteQAC
 from .pdqn import PDQN
-from .qac import QAC
 from .vac import VAC
 # algorithm-specific
 from .ppg import PPG
@@ -12,7 +12,9 @@ from .atoc import ATOC
 from .sqn import SQN
 from .acer import ACER
 from .qtran import QTran
-from .mappo import MAPPO
+from .mavac import MAVAC
 from .ngu import NGU
 from .qac_dist import QACDIST
+from .maqac import MAQAC, ContinuousMAQAC
 from .model_based import EnsembleDynamicsModel
+from .vae import VanillaVAE
