@@ -7,14 +7,17 @@ from .rainbow import RainbowDQNPolicy
 from .ddpg import DDPGPolicy
 from .d4pg import D4PGPolicy
 from .td3 import TD3Policy
+from .td3_vae import TD3VAEPolicy
+
 from .td3_bc import TD3BCPolicy
 from .a2c import A2CPolicy
 from .ppo import PPOPolicy
-from .sac import SACPolicy
+from .sac import SACPolicy, SACDiscretePolicy
 from .cql import CQLPolicy, CQLDiscretePolicy
 from .impala import IMPALAPolicy
 from .ngu import NGUPolicy
 from .r2d2 import R2D2Policy
+from .r2d2_gtrxl import R2D2GTrXLPolicy
 from .ppg import PPGPolicy
 from .sqn import SQNPolicy
 
