@@ -7,23 +7,21 @@ from torch.distributions import Independent, Normal
 
 from ding.torch_utils import Adam, to_device
 from ding.rl_utils import ppo_data, ppo_error, ppo_policy_error, ppo_policy_data, get_gae_with_default_last_value, \
-    v_nstep_td_data, v_nstep_td_error, get_nstep_return_data, get_train_sample, gae, gae_data, ppo_error_continuous,\
+    v_nstep_td_data, v_nstep_td_error, get_nstep_return_data, get_train_sample, gae, gae_data, ppo_error_continuous, \
     get_gae
 from ding.model import model_wrap
 from ding.utils import POLICY_REGISTRY, split_data_generator, RunningMeanStd
 from ding.utils.data import default_collate, default_decollate
 from .base_policy import Policy
 from .common_utils import default_preprocess_learn
+from ding.utils import dicts_to_lists, lists_to_dicts
 
 
 @POLICY_REGISTRY.register('ppo')
 class PPOPolicy(Policy):
     r"""
     Overview:
-        Policy class of PPO algorithm.
-
-        https://arxiv.org/pdf/1707.06347.pdf
-
+        Policy class of on policy version PPO algorithm.
     """
     config = dict(
         # (str) RL policy register name (refer to function "POLICY_REGISTRY").
@@ -34,10 +32,16 @@ class PPOPolicy(Policy):
         on_policy=True,
         # (bool) Whether to use priority(priority sample, IS weight, update priority)
         priority=False,
-        # (bool) Whether use Importance Sampling Weight to correct biased update. If True, priority must be True.
+        # (bool) Whether to use Importance Sampling Weight to correct biased update due to priority.
+        # If True, priority must be True.
         priority_IS_weight=False,
+        # (bool) Whether to recompurete advantages in each iteration of on-policy PPO
         recompute_adv=True,
-        continuous=True,
+        # (str) Which kind of action space used in PPOPolicy, ['discrete', 'continuous', 'hybrid']
+        action_space='discrete',
+        # (bool) Whether to use nstep return to calculate value target, otherwise, use return = adv + value
+        nstep_return=False,
+        # (bool) Whether to enable multi-agent training, i.e.: MAPPO
         multi_agent=False,
         # (bool) Whether to need policy data in process transition
         transition_with_policy_data=True,
@@ -90,16 +94,22 @@ class PPOPolicy(Policy):
         self._priority_IS_weight = self._cfg.priority_IS_weight
         assert not self._priority and not self._priority_IS_weight, "Priority is not implemented in PPO"
 
-        self._continuous = self._cfg.continuous
+        self._action_space = self._cfg.action_space
         if self._cfg.learn.ppo_param_init:
             for n, m in self._model.named_modules():
                 if isinstance(m, torch.nn.Linear):
                     torch.nn.init.orthogonal_(m.weight)
                     torch.nn.init.zeros_(m.bias)
-            if self._continuous:
+            if self._action_space in ['continuous', 'hybrid']:
                 # init log sigma
-                if hasattr(self._model.actor_head, 'log_sigma_param'):
-                    torch.nn.init.constant_(self._model.actor_head.log_sigma_param, -0.5)
+                if self._action_space == 'continuous':
+                    if hasattr(self._model.actor_head, 'log_sigma_param'):
+                        torch.nn.init.constant_(self._model.actor_head.log_sigma_param, -0.5)
+                elif self._action_space == 'hybrid':  # actor_head[1]: ReparameterizationHead, for action_args
+                    if hasattr(self._model.actor_head[1], 'log_sigma_param'):
+                        torch.nn.init.constant_(self._model.actor_head[1].log_sigma_param, -0.5)
+                        print('init ok')
+
                 for m in list(self._model.critic.modules()) + list(self._model.actor.modules()):
                     if isinstance(m, torch.nn.Linear):
                         # orthogonal initialization
@@ -156,27 +166,20 @@ class PPOPolicy(Policy):
         # ====================
         return_infos = []
         self._learn_model.train()
-        if self._value_norm:
-            unnormalized_return = data['adv'] + data['value'] * self._running_mean_std.std
-            data['return'] = unnormalized_return / self._running_mean_std.std
-            self._running_mean_std.update(unnormalized_return.cpu().numpy())
-        else:
-            data['return'] = data['adv'] + data['value']
 
         for epoch in range(self._cfg.learn.epoch_per_collect):
-            if self._recompute_adv:
+            if self._recompute_adv:  # new v network compute new value
                 with torch.no_grad():
-                    # obs = torch.cat([data['obs'], data['next_obs'][-1:]])
                     value = self._learn_model.forward(data['obs'], mode='compute_critic')['value']
                     next_value = self._learn_model.forward(data['next_obs'], mode='compute_critic')['value']
                     if self._value_norm:
                         value *= self._running_mean_std.std
                         next_value *= self._running_mean_std.std
 
-                    gae_data_ = gae_data(value, next_value, data['reward'], data['done'])
-                    # GAE need (T, B) shape input and return (T, B) output
-                    data['adv'] = gae(gae_data_, self._gamma, self._gae_lambda)
-                    # value = value[:-1]
+                    traj_flag = data.get('traj_flag', None)  # traj_flag indicates termination of trajectory
+                    compute_adv_data = gae_data(value, next_value, data['reward'], data['done'], traj_flag)
+                    data['adv'] = gae(compute_adv_data, self._gamma, self._gae_lambda)
+
                     unnormalized_returns = value + data['adv']
 
                     if self._value_norm:
@@ -187,6 +190,14 @@ class PPOPolicy(Policy):
                         data['value'] = value
                         data['return'] = unnormalized_returns
 
+            else:  # don't recompute adv
+                if self._value_norm:
+                    unnormalized_return = data['adv'] + data['value'] * self._running_mean_std.std
+                    data['return'] = unnormalized_return / self._running_mean_std.std
+                    self._running_mean_std.update(unnormalized_return.cpu().numpy())
+                else:
+                    data['return'] = data['adv'] + data['value']
+
             for batch in split_data_generator(data, self._cfg.learn.batch_size, shuffle=True):
                 output = self._learn_model.forward(batch['obs'], mode='compute_actor_critic')
                 adv = batch['adv']
@@ -195,18 +206,42 @@ class PPOPolicy(Policy):
                     adv = (adv - adv.mean()) / (adv.std() + 1e-8)
 
                 # Calculate ppo error
-                if self._continuous:
+                if self._action_space == 'continuous':
                     ppo_batch = ppo_data(
                         output['logit'], batch['logit'], batch['action'], output['value'], batch['value'], adv,
                         batch['return'], batch['weight']
                     )
                     ppo_loss, ppo_info = ppo_error_continuous(ppo_batch, self._clip_ratio)
-                else:
+                elif self._action_space == 'discrete':
                     ppo_batch = ppo_data(
                         output['logit'], batch['logit'], batch['action'], output['value'], batch['value'], adv,
                         batch['return'], batch['weight']
                     )
                     ppo_loss, ppo_info = ppo_error(ppo_batch, self._clip_ratio)
+                elif self._action_space == 'hybrid':
+                    # discrete part (discrete policy loss and entropy loss)
+                    ppo_discrete_batch = ppo_policy_data(
+                        output['logit']['action_type'], batch['logit']['action_type'], batch['action']['action_type'],
+                        adv, batch['weight']
+                    )
+                    ppo_discrete_loss, ppo_discrete_info = ppo_policy_error(ppo_discrete_batch, self._clip_ratio)
+                    # continuous part (continuous policy loss and entropy loss, value loss)
+                    ppo_continuous_batch = ppo_data(
+                        output['logit']['action_args'], batch['logit']['action_args'], batch['action']['action_args'],
+                        output['value'], batch['value'], adv, batch['return'], batch['weight']
+                    )
+                    ppo_continuous_loss, ppo_continuous_info = ppo_error_continuous(
+                        ppo_continuous_batch, self._clip_ratio
+                    )
+                    # sum discrete and continuous loss
+                    ppo_loss = type(ppo_continuous_loss)(
+                        ppo_continuous_loss.policy_loss + ppo_discrete_loss.policy_loss, ppo_continuous_loss.value_loss,
+                        ppo_continuous_loss.entropy_loss + ppo_discrete_loss.entropy_loss
+                    )
+                    ppo_info = type(ppo_continuous_info)(
+                        max(ppo_continuous_info.approx_kl, ppo_discrete_info.approx_kl),
+                        max(ppo_continuous_info.clipfrac, ppo_discrete_info.clipfrac)
+                    )
                 wv, we = self._value_weight, self._entropy_weight
                 total_loss = ppo_loss.policy_loss + wv * ppo_loss.value_loss - we * ppo_loss.entropy_loss
 
@@ -226,13 +261,13 @@ class PPOPolicy(Policy):
                     'value_max': output['value'].max().item(),
                     'approx_kl': ppo_info.approx_kl,
                     'clipfrac': ppo_info.clipfrac,
-                    'act': batch['action'].float().mean().item(),
                 }
-                if self._continuous:
+                if self._action_space == 'continuous':
                     return_info.update(
                         {
-                            'mu_mean': output['logit'][0].mean().item(),
-                            'sigma_mean': output['logit'][1].mean().item(),
+                            'act': batch['action'].float().mean().item(),
+                            'mu_mean': output['logit']['mu'].mean().item(),
+                            'sigma_mean': output['logit']['sigma'].mean().item(),
                         }
                     )
                 return_infos.append(return_info)
@@ -255,11 +290,13 @@ class PPOPolicy(Policy):
             Init traj and unroll length, collect model.
         """
         self._unroll_len = self._cfg.collect.unroll_len
-        self._continuous = self._cfg.continuous
-        if self._continuous:
-            self._collect_model = model_wrap(self._model, wrapper_name='base')
-        else:
+        self._action_space = self._cfg.action_space
+        if self._action_space == 'continuous':
+            self._collect_model = model_wrap(self._model, wrapper_name='reparam_sample')
+        elif self._action_space == 'discrete':
             self._collect_model = model_wrap(self._model, wrapper_name='multinomial_sample')
+        elif self._action_space == 'hybrid':
+            self._collect_model = model_wrap(self._model, wrapper_name='hybrid_reparam_multinomial_sample')
         self._collect_model.reset()
         self._gamma = self._cfg.collect.discount_factor
         self._gae_lambda = self._cfg.collect.gae_lambda
@@ -284,10 +321,6 @@ class PPOPolicy(Policy):
         self._collect_model.eval()
         with torch.no_grad():
             output = self._collect_model.forward(data, mode='compute_actor_critic')
-            if self._continuous:
-                (mu, sigma), value = output['logit'], output['value']
-                dist = Independent(Normal(mu, sigma), 1)
-                output['action'] = dist.sample()
         if self._cuda:
             output = to_device(output, 'cpu')
         output = default_decollate(output)
@@ -326,6 +359,10 @@ class PPOPolicy(Policy):
             - samples (:obj:`dict`): The training samples generated
         """
         data = to_device(data, self._device)
+        for transition in data:
+            transition['traj_flag'] = copy.deepcopy(transition['done'])
+        data[-1]['traj_flag'] = True
+
         if self._cfg.learn.ignore_done:
             data[-1]['done'] = False
 
@@ -375,11 +412,13 @@ class PPOPolicy(Policy):
             Evaluate mode init method. Called by ``self.__init__``.
             Init eval model with argmax strategy.
         """
-        self._continuous = self._cfg.continuous
-        if self._continuous:
-            self._eval_model = model_wrap(self._model, wrapper_name='base')
-        else:
+        self._action_space = self._cfg.action_space
+        if self._action_space == 'continuous':
+            self._eval_model = model_wrap(self._model, wrapper_name='deterministic_sample')
+        elif self._action_space == 'discrete':
             self._eval_model = model_wrap(self._model, wrapper_name='argmax_sample')
+        elif self._action_space == 'hybrid':
+            self._eval_model = model_wrap(self._model, wrapper_name='hybrid_deterministic_argmax_sample')
         self._eval_model.reset()
 
     def _forward_eval(self, data: dict) -> dict:
@@ -401,9 +440,6 @@ class PPOPolicy(Policy):
         self._eval_model.eval()
         with torch.no_grad():
             output = self._eval_model.forward(data, mode='compute_actor')
-            if self._continuous:
-                (mu, sigma) = output['logit']
-                output.update({'action': mu})
         if self._cuda:
             output = to_device(output, 'cpu')
         output = default_decollate(output)
@@ -411,7 +447,7 @@ class PPOPolicy(Policy):
 
     def default_model(self) -> Tuple[str, List[str]]:
         if self._cfg.multi_agent:
-            return 'mappo', ['ding.model.template.mappo']
+            return 'mavac', ['ding.model.template.mavac']
         else:
             return 'vac', ['ding.model.template.vac']
 
@@ -427,7 +463,7 @@ class PPOPolicy(Policy):
             'value_max',
             'value_mean',
         ]
-        if self._continuous:
+        if self._action_space == 'continuous':
             variables += ['mu_mean', 'sigma_mean', 'sigma_grad', 'act']
         return variables
 
@@ -664,25 +700,16 @@ class PPOOffPolicy(Policy):
         Returns:
                - transition (:obj:`dict`): Dict type transition data.
         """
-        if not self._nstep_return:
-            transition = {
-                'obs': obs,
-                'logit': model_output['logit'],
-                'action': model_output['action'],
-                'value': model_output['value'],
-                'reward': timestep.reward,
-                'done': timestep.done,
-            }
-        else:
-            transition = {
-                'obs': obs,
-                'next_obs': timestep.obs,
-                'logit': model_output['logit'],
-                'action': model_output['action'],
-                'value': model_output['value'],
-                'reward': timestep.reward,
-                'done': timestep.done,
-            }
+
+        transition = {
+            'obs': obs,
+            'next_obs': timestep.obs,
+            'logit': model_output['logit'],
+            'action': model_output['action'],
+            'value': model_output['value'],
+            'reward': timestep.reward,
+            'done': timestep.done,
+        }
         return transition
 
     def _get_train_sample(self, data: list) -> Union[None, List[Any]]:
