@@ -20,7 +20,6 @@ class R2D2Policy(Policy):
         Policy class of R2D2, from paper `Recurrent Experience Replay in Distributed Reinforcement Learning` .
         R2D2 proposes that several tricks should be used to improve upon DRQN,
         namely some recurrent experience replay tricks such as burn-in.
-
     Config:
         == ==================== ======== ============== ======================================== =======================
         ID Symbol               Type     Default Value  Description                              Other(Shape)
@@ -104,7 +103,7 @@ class R2D2Policy(Policy):
         ),
         collect=dict(
             # NOTE it is important that don't include key n_sample here, to make sure self._traj_len=INF
-            each_iter_n_sample=32,
+            # each_iter_n_sample=32,
             # `env_num` is used in hidden state, should equal to that one in env config.
             # User should specify this value in user config.
             env_num=None,
@@ -129,12 +128,9 @@ class R2D2Policy(Policy):
         r"""
         Overview:
             Init the learner model of R2D2Policy
-
         Arguments:
             .. note::
-
                 The _init_learn method takes the argument from the self._cfg.learn in the config file
-
             - learning_rate (:obj:`float`): The learning rate fo the optimizer
             - gamma (:obj:`float`): The discount factor
             - nstep (:obj:`int`): The num of n step return
@@ -182,10 +178,8 @@ class R2D2Policy(Policy):
         r"""
         Overview:
             Preprocess the data to fit the required data format for learning
-
         Arguments:
             - data (:obj:`List[Dict[str, Any]]`): the data collected from collect function
-
         Returns:
             - data (:obj:`Dict[str, Any]`): the processed data, including at least \
                 ['main_obs', 'target_obs', 'burnin_obs', 'action', 'reward', 'done', 'weight']
@@ -203,15 +197,15 @@ class R2D2Policy(Policy):
         else:
             data['weight'] = data.get('weight', None)
 
-        bs = self._burnin_step
+        burnin_step = self._burnin_step
 
         # data['done'], data['weight'], data['value_gamma'] is used in def _forward_learn() to calculate
         # the q_nstep_td_error, should be length of [self._unroll_len_add_burnin_step-self._burnin_step]
         ignore_done = self._cfg.learn.ignore_done
         if ignore_done:
-            data['done'] = [None for _ in range(self._unroll_len_add_burnin_step - bs)]
+            data['done'] = [None for _ in range(self._unroll_len_add_burnin_step - burnin_step)]
         else:
-            data['done'] = data['done'][bs:].float()  # for computation of online model self._learn_model
+            data['done'] = data['done'][burnin_step:].float()  # for computation of online model self._learn_model
             # NOTE that after the proprocessing of  get_nstep_return_data() in _get_train_sample
             # the data['done'] [t] is already the n-step done
 
@@ -219,27 +213,31 @@ class R2D2Policy(Policy):
         # with length of [self._unroll_len_add_burnin_step-self._burnin_step],
         # below is two different implementation ways
         if 'value_gamma' not in data:
-            data['value_gamma'] = [None for _ in range(self._unroll_len_add_burnin_step - bs)]
+            data['value_gamma'] = [None for _ in range(self._unroll_len_add_burnin_step - burnin_step)]
         else:
-            data['value_gamma'] = data['value_gamma'][bs:]
+            data['value_gamma'] = data['value_gamma'][burnin_step:]
 
         if 'weight' not in data or data['weight'] is None:
-            data['weight'] = [None for _ in range(self._unroll_len_add_burnin_step - bs)]
+            data['weight'] = [None for _ in range(self._unroll_len_add_burnin_step - burnin_step)]
         else:
             data['weight'] = data['weight'] * torch.ones_like(data['done'])
             # every timestep in sequence has same weight, which is the _priority_IS_weight in PER
 
-        data['action'] = data['action'][bs:-self._nstep]
-        data['reward'] = data['reward'][bs:-self._nstep]
+        # cut the seq_len from burn_in step to (seq_len - nstep) step
+        data['action'] = data['action'][burnin_step:-self._nstep]
+        # cut the seq_len from burn_in step to (seq_len - nstep) step
+        data['reward'] = data['reward'][burnin_step:-self._nstep]
 
         # the burnin_nstep_obs is used to calculate the init hidden state of rnn for the calculation of the q_value,
         # target_q_value, and target_q_action
-        data['burnin_nstep_obs'] = data['obs'][:bs + self._nstep]
+
+        # these slicing are all done in the outermost layer, which is the seq_len dim
+        data['burnin_nstep_obs'] = data['obs'][:burnin_step + self._nstep]
         # the main_obs is used to calculate the q_value, the [bs:-self._nstep] means using the data from
         # [bs] timestep to [self._unroll_len_add_burnin_step-self._nstep] timestep
-        data['main_obs'] = data['obs'][bs:-self._nstep]
+        data['main_obs'] = data['obs'][burnin_step:-self._nstep]
         # the target_obs is used to calculate the target_q_value
-        data['target_obs'] = data['obs'][bs + self._nstep:]
+        data['target_obs'] = data['obs'][burnin_step + self._nstep:]
 
         return data
 
@@ -248,21 +246,20 @@ class R2D2Policy(Policy):
         Overview:
             Forward and backward function of learn mode.
             Acquire the data, calculate the loss and optimize learner model.
-
         Arguments:
             - data (:obj:`dict`): Dict type data, including at least \
                 ['main_obs', 'target_obs', 'burnin_obs', 'action', 'reward', 'done', 'weight']
-
         Returns:
             - info_dict (:obj:`Dict[str, Any]`): Including cur_lr and total_loss
                 - cur_lr (:obj:`float`): Current learning rate
                 - total_loss (:obj:`float`): The calculated loss
         """
         # forward
-        data = self._data_preprocess_learn(data)
+        data = self._data_preprocess_learn(data)  # output datatype: Dict
         self._learn_model.train()
         self._target_model.train()
         # use the hidden state in timestep=0
+        # note the reset method is performed at the hidden state wrapper, to reset self._state.
         self._learn_model.reset(data_id=None, state=data['prev_state'][0])
         self._target_model.reset(data_id=None, state=data['prev_state'][0])
 
@@ -271,7 +268,8 @@ class R2D2Policy(Policy):
                 inputs = {'obs': data['burnin_nstep_obs'], 'enable_fast_timestep': True}
                 burnin_output = self._learn_model.forward(
                     inputs, saved_hidden_state_timesteps=[self._burnin_step, self._burnin_step + self._nstep]
-                )
+                )  # keys include 'logit', 'hidden_state' 'saved_hidden_state', \
+                # 'action', for their specific dim, please refer to DRQN model
                 burnin_output_target = self._target_model.forward(
                     inputs, saved_hidden_state_timesteps=[self._burnin_step, self._burnin_step + self._nstep]
                 )
@@ -307,6 +305,8 @@ class R2D2Policy(Policy):
             else:
                 l, e = q_nstep_td_error(td_data, self._gamma, self._nstep, value_gamma=value_gamma[t])
                 loss.append(l)
+                # td will be a list of the length (self._unroll_len_add_burnin_step - self._burnin_step - self._nstep)
+                # and each value is a tensor of the size batch_size
                 td_error.append(e.abs())
         loss = sum(loss) / (len(loss) + 1e-8)
 
@@ -314,6 +314,7 @@ class R2D2Policy(Policy):
         td_error_per_sample = 0.9 * torch.max(
             torch.stack(td_error), dim=0
         )[0] + (1 - 0.9) * (torch.sum(torch.stack(td_error), dim=0) / (len(td_error) + 1e-8))
+        # torch.max(torch.stack(td_error), dim=0) will return tuple like thing, please refer to torch.max
         # td_error shape list(<self._unroll_len_add_burnin_step-self._burnin_step-self._nstep>, B), for example, (75,64)
         # torch.sum(torch.stack(td_error), dim=0) can also be replaced with sum(td_error)
 
@@ -332,7 +333,7 @@ class R2D2Policy(Policy):
         return {
             'cur_lr': self._optimizer.defaults['lr'],
             'total_loss': loss.item(),
-            'priority': td_error_per_sample.abs().tolist(),
+            'priority': td_error_per_sample.tolist(),  # note abs operation has been performed above
             # the first timestep in the sequence, may not be the start of episode
             'q_s_taken-a_t0': q_s_a_t0.mean().item(),
             'target_q_s_max-a_t0': target_q_s_a_t0.mean().item(),
@@ -365,6 +366,8 @@ class R2D2Policy(Policy):
         self._unroll_len_add_burnin_step = self._cfg.unroll_len + self._cfg.burnin_step
         self._unroll_len = self._unroll_len_add_burnin_step  # for compatibility
 
+        # for r2d2, this hidden_state wrapper is to add the 'prev hidden state' for each transition.
+        # Note that collect env forms a batch and the key is added for the batch simultaneously.
         self._collect_model = model_wrap(
             self._model, wrapper_name='hidden_state', state_num=self._cfg.collect.env_num, save_prev_state=True
         )
@@ -427,10 +430,8 @@ class R2D2Policy(Policy):
         r"""
         Overview:
             Get the trajectory and the n step return data, then sample from the n_step return data
-
         Arguments:
             - data (:obj:`list`): The trajectory's cache
-
         Returns:
             - samples (:obj:`dict`): The training samples generated
         """
