@@ -8,3 +8,5 @@ from .soft_argmax import SoftArgmax
 from .transformer import Transformer
 from .scatter_connection import ScatterConnection
 from .resnet import resnet18, ResNet
+from .gumbel_softmax import GumbelSoftmax
+from .gtrxl import GTrXL
