@@ -125,7 +125,7 @@ def timestep_collate(batch: List[Dict[str, Any]]) -> Dict[str, Union[torch.Tenso
     prev_state = [b.pop('prev_state') for b in batch]
     batch_data = default_collate(batch)  # -> {some_key: T lists}, each list is [B, some_dim]
     batch_data = stack(batch_data)  # -> {some_key: [T, B, some_dim]}
-    batch_data['prev_state'] = list(zip(*prev_state))
+    batch_data['prev_state'] = list(zip(*prev_state))  # permute batch size dim with sequence len dim
     # append back prev_state, avoiding multi batch share the same data bug
     for i in range(len(batch)):
         batch[i]['prev_state'] = prev_state[i]
