@@ -8,8 +8,9 @@ lunarlander_r2d2_config = dict(
     env=dict(
         collector_env_num=collector_env_num,
         evaluator_env_num=evaluator_env_num,
+        env_id='LunarLander-v2',
         n_evaluator_episode=5,
-        stop_value=195,
+        stop_value=200,
     ),
     policy=dict(
         cuda=True,
@@ -52,12 +53,13 @@ lunarlander_r2d2_config = dict(
                 end=0.05,
                 decay=1e5,
             ),
-            replay_buffer=dict(replay_buffer_size=50000,
-                               # (Float type) How much prioritization is used: 0 means no prioritization while 1 means full prioritization
-                               alpha=0.6,
-                               # (Float type)  How much correction is used: 0 means no correction while 1 means full correction
-                               beta=0.4,
-                               )
+            replay_buffer=dict(
+                replay_buffer_size=50000,
+                # (Float type) How much prioritization is used: 0 means no prioritization while 1 means full prioritization
+                alpha=0.6,
+                # (Float type)  How much correction is used: 0 means no correction while 1 means full correction
+                beta=0.4,
+            )
         ),
     ),
 )
@@ -75,4 +77,4 @@ lunarlander_r2d2_create_config = EasyDict(lunarlander_r2d2_create_config)
 create_config = lunarlander_r2d2_create_config
 
 if __name__ == "__main__":
-    serial_pipeline([main_config, create_config], seed=0)
+    serial_pipeline([main_config, create_config], seed=0)
\ No newline at end of file
