@@ -6,6 +6,7 @@ from ding.envs import BaseEnv, BaseEnvTimestep, BaseEnvInfo
 from ding.envs.common.env_element import EnvElement, EnvElementInfo
 from ding.torch_utils import to_ndarray, to_list
 from ding.utils import ENV_REGISTRY
+from ding.envs.common import affine_transform
 
 
 @ENV_REGISTRY.register('lunarlander')
@@ -14,10 +15,21 @@ class LunarLanderEnv(BaseEnv):
     def __init__(self, cfg: dict) -> None:
         self._cfg = cfg
         self._init_flag = False
+        # env_id: LunarLander-v2, LunarLanderContinuous-v2
+        self._env_id = cfg.env_id
+        if 'Continuous' in self._env_id:
+            self._act_scale = cfg.act_scale  # act_scale only works in continous env
+        else:
+            self._act_scale = False
 
     def reset(self) -> np.ndarray:
         if not self._init_flag:
-            self._env = gym.make('LunarLander-v2')
+            self._env = gym.make(self._cfg.env_id)
+            self._observation_space = self._env.observation_space
+            self._action_space = self._env.action_space
+            self._reward_space = gym.spaces.Box(
+                low=self._env.reward_range[0], high=self._env.reward_range[1], shape=(1, ), dtype=np.float32
+            )
             self._init_flag = True
         if hasattr(self, '_seed') and hasattr(self, '_dynamic_seed') and self._dynamic_seed:
             np_seed = 100 * np.random.randint(1, 1000)
@@ -45,7 +57,9 @@ class LunarLanderEnv(BaseEnv):
     def step(self, action: np.ndarray) -> BaseEnvTimestep:
         assert isinstance(action, np.ndarray), type(action)
         if action.shape == (1, ):
-            action = action.squeeze()  # 0-dim array
+            action = action.item()  # 0-dim array
+        if self._act_scale:
+            action = affine_transform(action, min_val=-1, max_val=1)
         obs, rew, done, info = self._env.step(action)
         # self._env.render()
         rew = float(rew)
@@ -56,41 +70,6 @@ class LunarLanderEnv(BaseEnv):
         rew = to_ndarray([rew])  # wrapped to be transfered to a array with shape (1,)
         return BaseEnvTimestep(obs, rew, done, info)
 
-    def info(self) -> BaseEnvInfo:
-        T = EnvElementInfo
-        return BaseEnvInfo(
-            agent_num=1,
-            obs_space=T(
-                (8, ),
-                {
-                    'min': [float("-inf")] * 8,
-                    'max': [float("inf")] * 8,
-                    'dtype': np.float32,
-                },
-            ),
-            # [min, max)
-            act_space=T(
-                (1, ),
-                {
-                    'min': 0,
-                    'max': 4,
-                    'dtype': int,
-                },
-            ),
-            rew_space=T(
-                (1, ),
-                {
-                    'min': -1000.0,
-                    'max': 1000.0,
-                    'dtype': np.float32,
-                },
-            ),
-            use_wrappers=None,
-        )
-
-    def __repr__(self) -> str:
-        return "DI-engine LunarLander Env"
-
     def enable_save_replay(self, replay_path: Optional[str] = None) -> None:
         if replay_path is None:
             replay_path = './video'
@@ -99,3 +78,26 @@ class LunarLanderEnv(BaseEnv):
         self._env = gym.wrappers.Monitor(
             self._env, self._replay_path, video_callable=lambda episode_id: True, force=True
         )
+
+    def random_action(self) -> np.ndarray:
+        random_action = self.action_space.sample()
+        if isinstance(random_action, np.ndarray):
+            pass
+        elif isinstance(random_action, int):
+            random_action = to_ndarray([random_action], dtype=np.int64)
+        return random_action
+
+    @property
+    def observation_space(self) -> gym.spaces.Space:
+        return self._observation_space
+
+    @property
+    def action_space(self) -> gym.spaces.Space:
+        return self._action_space
+
+    @property
+    def reward_space(self) -> gym.spaces.Space:
+        return self._reward_space
+
+    def __repr__(self) -> str:
+        return "DI-engine LunarLander Env"
\ No newline at end of file
