@@ -16,6 +16,7 @@ cartpole_ppo_offpolicy_config = dict(
             encoder_hidden_size_list=[64, 64, 128],
             critic_head_hidden_size=128,
             actor_head_hidden_size=128,
+            action_space='discrete',
         ),
         learn=dict(
             update_per_collect=6,
@@ -24,6 +25,7 @@ cartpole_ppo_offpolicy_config = dict(
             value_weight=0.5,
             entropy_weight=0.01,
             clip_ratio=0.2,
+            learner=dict(hook=dict(save_ckpt_after_iter=1000)),
         ),
         collect=dict(
             n_sample=128,
@@ -31,6 +33,7 @@ cartpole_ppo_offpolicy_config = dict(
             discount_factor=0.9,
             gae_lambda=0.95,
         ),
+        eval=dict(evaluator=dict(eval_freq=40, )),
         other=dict(replay_buffer=dict(replay_buffer_size=5000))
     ),
 )
