@@ -19,8 +19,13 @@ cartpole_sqil_config = dict(
         nstep=1,
         discount_factor=0.97,
         learn=dict(batch_size=64, learning_rate=0.001, alpha=0.12),
-        # Users should add their own path here (path should lead to a well-trained model)
-        collect=dict(n_sample=8, demonstration_info_path='path'),
+        collect=dict(
+            n_sample=8,
+            # Users should add their own model path here. Model path should lead to a model.
+            # Absolute path is recommended.
+            # In DI-engine, it is ``exp_name/ckpt/ckpt_best.pth.tar``.
+            model_path='model_path_placeholder'
+        ),
         # note: this is the times after which you learns to evaluate
         eval=dict(evaluator=dict(eval_freq=50, )),
         other=dict(
