@@ -13,10 +13,11 @@ league_demo_ppo_config = dict(
     ),
     policy=dict(
         cuda=False,
-        continuous=False,
+        action_space='discrete',
         model=dict(
             obs_shape=2,
             action_shape=2,
+            action_space='discrete',
             encoder_hidden_size_list=[32, 32],
             critic_head_hidden_size=32,
             actor_head_hidden_size=32,
@@ -38,6 +39,7 @@ league_demo_ppo_config = dict(
                 patience=50,
                 # cooldown=0,
             ),
+            learner=dict(log_policy=False),
         ),
         collect=dict(
             n_episode=128, unroll_len=1, discount_factor=1.0, gae_lambda=1.0, collector=dict(get_train_sample=True, )
@@ -55,7 +57,7 @@ league_demo_ppo_config = dict(
                     one_phase_step=200,
                     branch_probs=dict(
                         pfsp=0.5,
-                        sp=1.0,
+                        sp=0.5,
                     ),
                     strong_win_rate=0.7,
                 ),
