@@ -6,12 +6,13 @@ import torch
 from tensorboardX import SummaryWriter
 
 from ding.config import compile_config
-from ding.worker import BaseLearner, BattleEpisodeSerialCollector, BattleInteractionSerialEvaluator, NaiveReplayBuffer
+from ding.worker import BaseLearner, BattleInteractionSerialEvaluator, NaiveReplayBuffer
 from ding.envs import BaseEnvManager, DingEnvWrapper
 from ding.policy import PPOPolicy
 from ding.model import VAC
 from ding.utils import set_pkg_seed
 from dizoo.league_demo.game_env import GameEnv
+from dizoo.league_demo.league_demo_collector import LeagueDemoCollector
 from dizoo.league_demo.league_demo_ppo_config import league_demo_ppo_config
 
 
@@ -38,14 +39,14 @@ class EvalPolicy2:
         pass
 
 
-def main(cfg, seed=0, max_iterations=int(1e10)):
+def main(cfg, seed=0, max_train_iter=int(1e8), max_env_step=int(1e8)):
     cfg.exp_name = 'selfplay_demo_ppo'
     cfg = compile_config(
         cfg,
         BaseEnvManager,
         PPOPolicy,
         BaseLearner,
-        BattleEpisodeSerialCollector,
+        LeagueDemoCollector,
         BattleInteractionSerialEvaluator,
         NaiveReplayBuffer,
         save_cfg=True
@@ -81,7 +82,7 @@ def main(cfg, seed=0, max_iterations=int(1e10)):
     learner2 = BaseLearner(
         cfg.policy.learn.learner, policy2.learn_mode, tb_logger, exp_name=cfg.exp_name, instance_name='learner2'
     )
-    collector = BattleEpisodeSerialCollector(
+    collector = LeagueDemoCollector(
         cfg.policy.collect.collector,
         collector_env, [policy1.collect_mode, policy2.collect_mode],
         tb_logger,
@@ -107,13 +108,11 @@ def main(cfg, seed=0, max_iterations=int(1e10)):
         instance_name='uniform_evaluator'
     )
 
-    for _ in range(max_iterations):
+    while True:
         if evaluator1.should_eval(learner1.train_iter):
-            stop_flag1, reward, _ = evaluator1.eval(learner1.save_checkpoint, learner1.train_iter, collector.envstep)
-            tb_logger.add_scalar('fixed_evaluator_step/reward_mean', reward, collector.envstep)
+            stop_flag1, _ = evaluator1.eval(learner1.save_checkpoint, learner1.train_iter, collector.envstep)
         if evaluator2.should_eval(learner1.train_iter):
-            stop_flag2, reward, _ = evaluator2.eval(learner1.save_checkpoint, learner1.train_iter, collector.envstep)
-            tb_logger.add_scalar('uniform_evaluator_step/reward_mean', reward, collector.envstep)
+            stop_flag2, _ = evaluator2.eval(learner1.save_checkpoint, learner1.train_iter, collector.envstep)
         if stop_flag1 and stop_flag2:
             break
         train_data, _ = collector.collect(train_iter=learner1.train_iter)
@@ -123,6 +122,8 @@ def main(cfg, seed=0, max_iterations=int(1e10)):
         for i in range(cfg.policy.learn.update_per_collect):
             learner1.train(train_data[0], collector.envstep)
             learner2.train(train_data[1], collector.envstep)
+        if collector.envstep >= max_env_step or learner1.train_iter >= max_train_iter:
+            break
 
 
 if __name__ == "__main__":
