@@ -2,14 +2,14 @@ from typing import Dict
 import gym
 import numpy as np
 
-from ding.envs import ObsNormEnv, RewardNormEnv
+from ding.envs import ObsNormWrapper, RewardNormWrapper, DelayRewardWrapper, FinalEvalRewardEnv
 
 
 def wrap_mujoco(
         env_id,
         norm_obs: Dict = dict(use_norm=False, ),
         norm_reward: Dict = dict(use_norm=False, ),
-        only_info=False
+        delay_reward_step: int = 1
 ) -> gym.Env:
     r"""
     Overview:
@@ -22,17 +22,13 @@ def wrap_mujoco(
     Returns:
         - wrapped_env (:obj:`gym.Env`): The wrapped mujoco environment
     """
-    if not only_info:
-        env = gym.make(env_id)
-        if norm_obs is not None and norm_obs.use_norm:
-            env = ObsNormEnv(env)
-        if norm_reward is not None and norm_reward.use_norm:
-            env = RewardNormEnv(env, norm_reward.reward_discount)
-        return env
-    else:
-        wrapper_info = ''
-        if norm_obs is not None and norm_obs.use_norm:
-            wrapper_info = ObsNormEnv.__name__ + '\n'
-        if norm_reward is not None and norm_reward.use_norm:
-            wrapper_info += RewardNormEnv.__name__ + '\n'
-        return wrapper_info
+    env = gym.make(env_id)
+    env = FinalEvalRewardEnv(env)
+    if norm_obs is not None and norm_obs.use_norm:
+        env = ObsNormWrapper(env)
+    if norm_reward is not None and norm_reward.use_norm:
+        env = RewardNormWrapper(env, norm_reward.reward_discount)
+    if delay_reward_step > 1:
+        env = DelayRewardWrapper(env, delay_reward_step)
+
+    return env
